import { Config, LogicalNodeGenerator, LayoutResult, Display, LogicalClear, LogicalFloat, InlineMargin, BlockMargin, PageRootFormatContext, LineReducer, BlockReducer, WhiteSpace, } from './public-api';
export class BlockNodeGenerator {
    constructor(context, blockReducer = BlockReducer.instance, lineReducer = LineReducer.instance) {
        this.context = context;
        this.blockReducer = blockReducer;
        this.lineReducer = lineReducer;
        this.generator = this.createGenerator();
    }
    getNext() {
        const next = this.generator.next();
        return next.done ? undefined : next.value;
    }
    *createGenerator() {
        if (Config.debugLayout) {
            console.group(`block: ${this.context.name}`);
        }
        this.context.flowRoot.openElement(this.context.env.element);
        const progressDelta = 1 / this.context.env.element.childNodes.length;
        const isPageRoot = this.context instanceof PageRootFormatContext;
        let prevChildGen = undefined;
        if (this.context.env.pageBreakBefore.isAlways()) {
            yield LayoutResult.pageBreak(this.context, "block-fmt-context: page-break-before");
        }
        if (this.context.env.measure && this.context.env.measure <= 0) {
            yield LayoutResult.skip(this.context, "Minus size measure");
            return;
        }
        if (!isPageRoot && this.context.restExtent < this.context.env.edge.borderBoxBefore) {
            yield LayoutResult.pageBreak(this.context, "block-fmt-context: before border can't be included");
        }
        this.context.addBorderBoxEdge("before");
        this.context.addBorderBoxEdge("start");
        this.context.addBorderBoxEdge("end");
        let childElement = this.context.env.element.firstChild;
        while (childElement !== null) {
            const display = Display.load(childElement);
            const whiteSpace = WhiteSpace.load(childElement);
            if (display.isNone() || (!whiteSpace.isPre() && WhiteSpace.isWhiteSpaceElement(childElement))) {
                childElement = childElement.nextSibling;
                continue;
            }
            const float = LogicalFloat.load(childElement);
            if ((!float.isNone() || display.isBlockLevel()) && this.context.inlineNodes.length > 0) {
                const line = this.context.acceptLayoutReducer(this.lineReducer);
                this.context.addLine(line.body);
            }
            this.context.curChildStartMargin = InlineMargin.getMarginFromParentBlock(childElement);
            const childGen = LogicalNodeGenerator.createChild(childElement, this.context);
            this.context.child = childGen.generator;
            const clear = LogicalClear.load(childElement);
            if (!clear.isNone() && this.context.flowRoot.floatRegion) {
                const clearedExtent = this.context.flowRoot.clearFloat(clear);
                if (Config.debugLayout) {
                    console.log("float clearance:%d (cur before = %d)", clearedExtent, this.context.cursorPos.before);
                }
                this.context.cursorPos.before = Math.max(this.context.cursorPos.before, clearedExtent);
            }
            const beforeMargin = BlockMargin.getFlowMarginFromLastElement(this.context.env, this.context.child, prevChildGen);
            if (this.context.restExtent < beforeMargin) {
                yield this.context.acceptLayoutReducer(this.blockReducer);
                if (!isPageRoot) {
                    yield LayoutResult.pageBreak(this.context, `block-fmt-context: before-margin(${beforeMargin}) overflow.`);
                }
            }
            if (beforeMargin > 0) {
                this.context.addBlockMarginEdge("before", beforeMargin);
            }
            while (true) {
                this.context.suspendedGens = this.context.suspendedGens.filter(generator => {
                    const fvalue = generator.getNext();
                    if (!fvalue) {
                        return false;
                    }
                    if (fvalue.type === "block") {
                        const float = generator.context.env.float;
                        this.context.setFloat(fvalue.body, float);
                    }
                    return true;
                });
                const value = this.context.child.getNext();
                if (!value) {
                    break;
                }
                if (!float.isNone() && value.isFloatable()) {
                    this.context.setFloat(value.body, float);
                    this.context.suspendedGens.push(this.context.child);
                    break;
                }
                if (value.type === 'skip') {
                    break;
                }
                if (value.type === 'line-break') {
                    const isBr = value.body.env.element.tagName === "br";
                    const line = this.context.acceptLayoutReducer(this.lineReducer, isBr);
                    this.context.addLine(line.body);
                }
                else if (value.type === 'iblock-inline-break') {
                    if (this.context.env.display.isInlineBlockFlow()) {
                        const line = this.context.acceptLayoutReducer(this.lineReducer, false);
                        if (line.body.size.hasZero()) {
                            yield value;
                        }
                        else {
                            this.context.addLine(line.body);
                        }
                    }
                    else {
                        const line = this.context.acceptLayoutReducer(this.lineReducer, false);
                        this.context.addLine(line.body);
                    }
                }
                else if (value.type === 'page-break') {
                    if (this.context.inlineNodes.length > 0) {
                        const line = this.context.acceptLayoutReducer(this.lineReducer, false);
                        this.context.addLine(line.body);
                    }
                    const block = this.context.acceptLayoutReducer(this.blockReducer);
                    if (isPageRoot) {
                        if (block.body.text.trim() !== "") {
                            yield block;
                        }
                    }
                    else {
                        yield block;
                        yield value;
                    }
                }
                else if (value.type === 'block') {
                    this.context.addBlock(value.body);
                }
                else if (value.type === 'inline-block') {
                    this.context.addInlineBlock(value.body);
                }
                else if (value.type === 'table') {
                    this.context.addTable(value.body);
                }
                else if (value.type === 'table-row-group') {
                    this.context.addTableRowGroup(value.body);
                }
                else if (value.type === 'table-row') {
                    this.context.addTableRow(value.body);
                }
                else if (value.type === 'table-cells') {
                    this.context.addTableCells(value.body);
                }
                else if (value.type === 'inline') {
                    this.context.addInline(value.body);
                }
                else if (value.type === 'list-marker') {
                    this.context.addListMarker(value.body);
                }
                else if (value.type === 'text') {
                    const textNode = value.body;
                    if (textNode.skipBr && childGen.nextElement && childGen.nextElement.tagName === "br") {
                        childGen.nextElement = childGen.nextElement.nextSibling;
                    }
                    this.context.addText(textNode);
                }
                else if (value.type === 'ruby') {
                    this.context.addRuby(value.body);
                }
                else if (value.type === 're-block') {
                    this.context.addBlockRe(value.body);
                }
                else if (value.type === 're-inline') {
                    this.context.addInlineRe(value.body);
                }
                else if (value.type === 'inline-link') {
                    this.context.addInlineLink(value.body);
                }
                else if (value.type === 'block-link') {
                    this.context.addBlockLink(value.body);
                }
            }
            this.context.progress = (childElement.index + 1) * progressDelta;
            prevChildGen = childGen.generator;
            childElement = childGen.nextElement;
        }
        if (this.context.inlineNodes.length > 0) {
            const line = this.context.acceptLayoutReducer(this.lineReducer, false);
            this.context.addLine(line.body);
        }
        if (!isPageRoot && this.context.restExtent < this.context.contextBoxEdge.getBorderBoxEdgeSize("after")) {
            yield LayoutResult.pageBreak(this.context, "block-fmt-context: after border can't be included");
        }
        this.context.addBorderBoxEdge("after");
        const marginAfter = this.context.contextBoxEdge.margin.getSize("after");
        if (!this.context.env.float.isNone() && this.context.restExtent >= marginAfter) {
            this.context.addBlockMarginEdge("after", marginAfter);
        }
        this.context.flowRoot.closeElement(this.context.env.element);
        yield this.context.acceptLayoutReducer(this.blockReducer);
        if (!isPageRoot && this.context.env.pageBreakAfter.isAlways()) {
            yield LayoutResult.pageBreak(this.context, "block-fmt-context: page-break-after");
        }
        if (Config.debugLayout) {
            console.groupEnd();
        }
    }
}
//# sourceMappingURL=block-node-generator.js.map