import { Selector, Specificity } from "./public-api";
export class ComplexSelector extends Selector {
    constructor(selectors, combinators) {
        super();
        this.selectors = selectors;
        this.combinators = combinators;
        this.specificity = this.getSpecificity();
    }
    getSelectorItem(index) {
        if (index < 0 || index >= this.selectors.length) {
            throw new Error("getSelectorItem: out of range");
        }
        return this.selectors[index];
    }
    getCombinatorItem(index) {
        if (index < 0 || index >= this.combinators.length) {
            throw new Error("getCombinatorItem: out of range");
        }
        return this.combinators[index];
    }
    static compare(selector1, selector2) {
        return Specificity.compare(selector1.specificity, selector2.specificity);
    }
    getSpecificity() {
        let specificity = new Specificity(0, 0, 0);
        specificity = this.selectors.reduce((acm, selector) => {
            return Specificity.add(acm, selector.specificity);
        }, specificity);
        return specificity;
    }
    get leafSelector() {
        return this.selectors[0];
    }
    get peSelector() {
        for (let i = 0; i < this.selectors.length; i++) {
            let selector = this.selectors[i];
            if (selector.pseudoElement !== null) {
                return selector.pseudoElement;
            }
        }
        return null;
    }
    toString() {
        let str = this.leafSelector.toString();
        for (var spos = 1, cpos = 0; spos < this.selectors.length; spos++, cpos++) {
            str = this.selectors[spos].toString() + this.combinators[cpos] + str;
        }
        return str;
    }
    queryDirectParent(element, parent_sel) {
        if (element.parent && parent_sel.test(element.parent)) {
            return element.parent;
        }
        return null;
    }
    queryParent(element, selector) {
        let parent = element.parent;
        while (parent) {
            if (selector.test(parent)) {
                return parent;
            }
            parent = parent.parent;
        }
        return parent;
    }
    queryDirectSibling(element, selector) {
        let prev_element = element.previousSibling;
        if (prev_element && selector.test(prev_element)) {
            return prev_element;
        }
        return null;
    }
    querySibling(element, prev_selector) {
        let prev = element.previousSibling;
        while (prev) {
            if (!prev.isTextElement() && prev_selector.test(prev)) {
                return prev;
            }
            prev = prev.previousSibling;
        }
        return prev;
    }
    queryLeft(left, cmb, elm) {
        switch (cmb) {
            case " ":
                return this.queryParent(elm, left);
            case ">":
                return this.queryDirectParent(elm, left);
            case "+":
                return this.queryDirectSibling(elm, left);
            case "~":
                return this.querySibling(elm, left);
            default:
                throw new Error("Invalid combinator[" + cmb + "]");
        }
    }
    querySelectorAll(element) {
        let leaf_elements = this.leafSelector.querySelectorAll(element);
        return leaf_elements.filter((elm) => {
            return this.test(elm);
        });
    }
    querySelector(element) {
        let leaf_elements = this.leafSelector.querySelectorAll(element);
        for (let i = 0; i < leaf_elements.length; i++) {
            let elm = leaf_elements[i];
            if (this.test(elm)) {
                return elm;
            }
        }
        return null;
    }
    test(element, matchAsPeOwner = false) {
        let spos = 0, cpos = 0;
        let slen = this.selectors.length, clen = this.combinators.length;
        let cur = element;
        while (true) {
            if (spos >= slen) {
                break;
            }
            let left = this.selectors[spos];
            if (spos === 0 && !left.test(cur, matchAsPeOwner)) {
                return false;
            }
            if (cpos >= clen) {
                break;
            }
            let cmb = this.combinators[cpos++];
            if (spos + 1 >= slen) {
                return false;
            }
            left = this.selectors[spos + 1];
            cur = this.queryLeft(left, cmb, cur);
            if (cur === null) {
                return false;
            }
            spos++;
        }
        return true;
    }
}
//# sourceMappingURL=complex-selector.js.map