import { Config, ContainingElement, CssCascade, LogicalSize, LogicalPadding, LogicalBorder, BoxSizing, } from './public-api';
function loadOptionalNumber(element, prop) {
    const value = CssCascade.getValue(element, prop);
    return (value === "none") ? value : parseInt(value, 10);
}
function loadAutableNumber(element, prop) {
    const value = CssCascade.getValue(element, prop);
    return (value === "auto") ? value : parseInt(value, 10);
}
class ComputedLength {
    constructor(length, prop) {
        this.length = length;
        this.prop = prop;
    }
    static load(element, prop) {
        const value = loadAutableNumber(element, prop);
        return new ComputedLength(value, prop);
    }
    get number() {
        if (this.length === "auto") {
            throw new Error("length is not resolved.");
        }
        return this.length;
    }
    set number(value) {
        this.length = value;
    }
    isAuto() {
        return this.length === "auto";
    }
    setAutoZero() {
        this.length = (this.length === "auto") ? 0 : this.length;
    }
    save(style) {
        if (this.length !== "auto") {
            style.setProperty(this.prop, this.length + "px");
        }
    }
}
class ComputedPosition {
    constructor(before, end, after, start) {
        this.before = before;
        this.end = end;
        this.after = after;
        this.start = start;
    }
    static load(element) {
        return new ComputedPosition(ComputedLength.load(element, "before"), ComputedLength.load(element, "end"), ComputedLength.load(element, "after"), ComputedLength.load(element, "start"));
    }
    save(style) {
        this.before.save(style);
        this.end.save(style);
        this.after.save(style);
        this.start.save(style);
    }
    isAutoInline() {
        return this.start.isAuto() && this.end.isAuto();
    }
}
class ComputedMinSize {
    constructor(minSize, prop) {
        this.minSize = minSize;
        this.prop = prop;
    }
    static load(element, prop) {
        return new ComputedMinSize(loadOptionalNumber(element, prop), prop);
    }
    save(style) {
        if (this.minSize !== "none") {
            style.setProperty(this.prop, this.minSize + "px");
        }
    }
    apply(value) {
        return this.minSize !== "none" ? Math.max(this.minSize, value) : value;
    }
}
class ComputedMaxSize {
    constructor(maxSize, prop) {
        this.maxSize = maxSize;
        this.prop = prop;
    }
    static load(element, prop) {
        return new ComputedMaxSize(loadOptionalNumber(element, prop), prop);
    }
    save(style) {
        if (this.maxSize !== "none") {
            style.setProperty(this.prop, this.maxSize + "px");
        }
    }
    apply(value) {
        return this.maxSize !== "none" ? Math.min(this.maxSize, value) : value;
    }
}
class ComputedMinMaxRange {
    constructor(minSize, maxSize) {
        this.minSize = minSize;
        this.maxSize = maxSize;
    }
    static load(element, minProp, maxProp) {
        return new ComputedMinMaxRange(ComputedMinSize.load(element, minProp), ComputedMaxSize.load(element, maxProp));
    }
    save(style) {
        this.minSize.save(style);
        this.maxSize.save(style);
    }
    apply(value) {
        return this.maxSize.apply(this.minSize.apply(value));
    }
}
class ComputedMinMaxBoxSize {
    constructor(minMaxMeasure, minMaxExtent) {
        this.minMaxMeasure = minMaxMeasure;
        this.minMaxExtent = minMaxExtent;
    }
    static load(element) {
        return new ComputedMinMaxBoxSize(ComputedMinMaxRange.load(element, "min-measure", "max-measure"), ComputedMinMaxRange.load(element, "min-extent", "max-extent"));
    }
    save(style) {
        this.minMaxMeasure.save(style);
        this.minMaxExtent.save(style);
    }
    apply(measure, extent) {
        return new LogicalSize({
            measure: this.minMaxMeasure.apply(measure),
            extent: this.minMaxExtent.apply(extent)
        });
    }
}
class ComputedLogicalSize {
    constructor(measure, extent) {
        this.measure = measure;
        this.extent = extent;
    }
    static load(element) {
        let measure = ComputedLength.load(element, "measure");
        let extent = ComputedLength.load(element, "extent");
        if (!element.parent) {
            measure = measure.isAuto() ? new ComputedLength(Config.defaultBodyMeasure, "measure") : measure;
            extent = extent.isAuto() ? new ComputedLength(Config.defaultBodyExtent, "extent") : extent;
        }
        return new ComputedLogicalSize(measure, extent);
    }
    save(style) {
        this.measure.save(style);
        this.extent.save(style);
    }
}
class ComputedPhysicalSize {
    constructor(width, height) {
        this.width = width;
        this.height = height;
    }
    static load(element) {
        const width = this.loadPhysicalSize(element, "width");
        const height = this.loadPhysicalSize(element, "height");
        return new ComputedPhysicalSize(width, height);
    }
    static loadPhysicalSize(element, prop) {
        let attrSize = element.getAttribute(prop);
        if (attrSize) {
            return new ComputedLength(parseInt(attrSize), prop);
        }
        return ComputedLength.load(element, prop);
    }
    save(style) {
        this.width.save(style);
        this.height.save(style);
    }
}
class ComputedMargin {
    constructor(before, end, after, start) {
        this.before = before;
        this.end = end;
        this.after = after;
        this.start = start;
    }
    static load(element) {
        return new ComputedMargin(ComputedLength.load(element, "margin-before"), ComputedLength.load(element, "margin-end"), ComputedLength.load(element, "margin-after"), ComputedLength.load(element, "margin-start"));
    }
    save(style) {
        this.before.save(style);
        this.end.save(style);
        this.after.save(style);
        this.start.save(style);
    }
    get measure() {
        return this.start.number + this.end.number;
    }
    get extent() {
        return this.before.number + this.after.number;
    }
    isAutoMeasure() {
        return this.start.isAuto() || this.end.isAuto();
    }
    isAutoExtent() {
        return this.before.isAuto() || this.after.isAuto();
    }
    clearAutoInline() {
        this.start.setAutoZero();
        this.end.setAutoZero();
    }
    clearAutoBlock() {
        this.before.setAutoZero();
        this.after.setAutoZero();
    }
    clearInline() {
        this.start.number = 0;
        this.end.number = 0;
    }
    clearBlock() {
        this.before.number = 0;
        this.after.number = 0;
    }
}
class ComputedBoxEdges {
    constructor(padding, border, margin) {
        this.padding = padding;
        this.border = border;
        this.margin = margin;
    }
    static load(element) {
        return new ComputedBoxEdges(LogicalPadding.load(element), LogicalBorder.load(element), ComputedMargin.load(element));
    }
    save(style) {
        this.margin.save(style);
    }
    get measure() {
        return this.borderBoxMeasure + this.margin.measure;
    }
    get extent() {
        return this.borderBoxExtent + this.margin.extent;
    }
    get borderBoxMeasure() {
        return this.border.width.measure + this.padding.measure;
    }
    get borderBoxExtent() {
        return this.border.width.extent + this.padding.extent;
    }
}
export class ComputedRegion {
    constructor(boxSizing, containingMeasure, containingExtent, logicalSize, physicalSize, position, edges, minMaxBoxSize) {
        this.boxSizing = boxSizing;
        this.containingMeasure = containingMeasure;
        this.containingExtent = containingExtent;
        this.logicalSize = logicalSize;
        this.physicalSize = physicalSize;
        this.position = position;
        this.edges = edges;
        this.minMaxBoxSize = minMaxBoxSize;
    }
    static load(element) {
        const containingElement = ContainingElement.get(element);
        const containingMeasure = ComputedLength.load(containingElement, "measure");
        const containingExtent = ComputedLength.load(containingElement, "extent");
        if (containingMeasure.isAuto()) {
            return undefined;
        }
        return new ComputedRegion(BoxSizing.load(element), containingMeasure.number, containingExtent, ComputedLogicalSize.load(element), ComputedPhysicalSize.load(element), ComputedPosition.load(element), ComputedBoxEdges.load(element), ComputedMinMaxBoxSize.load(element));
    }
    save(style) {
        this.logicalSize.save(style);
        this.physicalSize.save(style);
        this.position.save(style);
        this.edges.save(style);
        this.minMaxBoxSize.save(style);
    }
    get borderBoxMeasure() {
        if (this.logicalSize.measure.length === "auto") {
            throw new Error("measure is not resolved.");
        }
        if (this.boxSizing.isBorderBox()) {
            return this.logicalSize.measure.length;
        }
        if (this.boxSizing.isPaddingBox()) {
            return this.logicalSize.measure.length + this.edges.border.width.measure;
        }
        return this.logicalSize.measure.length + this.edges.borderBoxMeasure;
    }
}
//# sourceMappingURL=computed-value.js.map