import { Config, CssStyleDeclaration, CssRule, CssProp, CssText, CssMacro, SelectorText, SelectorParser, SelectorLexer, DynamicStyle, DomCallback, Font, ListStyle, LogicalMargin, LogicalPadding, LogicalBorder, LogicalBorderWidth, LogicalBorderColor, LogicalBorderStyle, LogicalBorderRadius, TextEmphasis } from "./public-api";
export class CssParser {
    static parseInlineStyle(inlineStyle) {
        return CssText.normalize(inlineStyle).split(";").reduce((acm, stext) => {
            const nvs = stext.split(":");
            if (nvs.length === 2 && nvs[0] !== "" && nvs[1] !== "") {
                const cssProp = new CssProp(nvs[0]);
                if (Config.IgnoredInlineStyleProps.indexOf(cssProp.value) >= 0) {
                    return acm;
                }
                const cssText = new CssText({ prop: cssProp.value, value: nvs[1] });
                CssParser.parseDeclaration(cssProp, cssText).forEach((declr) => {
                    acm.setProperty(declr.prop, declr.value);
                });
            }
            return acm;
        }, new CssStyleDeclaration());
    }
    static parseRule(selectorSrc, declrBlock) {
        return new SelectorText(selectorSrc).split().map((stext) => {
            const lexer = new SelectorLexer(stext);
            const selector = new SelectorParser(lexer).parse();
            const declrs = this.parseDeclarationBlock(stext, declrBlock);
            return new CssRule(selector, declrs);
        });
    }
    static parseDeclarationBlock(selector, declrBlock) {
        return Object.keys(declrBlock).reduce((acm, prop) => {
            const objValue = declrBlock[prop];
            const cssProp = new CssProp(prop);
            if (typeof objValue === "string") {
                return acm.setProperty(cssProp.value, objValue);
            }
            if (typeof objValue === "number") {
                return acm.setProperty(cssProp.value, String(objValue));
            }
            if (typeof objValue === "function") {
                if (cssProp.isDynamicStyleProp()) {
                    const name = cssProp.getDynamicStylePropName();
                    const callback = objValue;
                    return acm.addDynamicStyle(new DynamicStyle(selector, name, callback));
                }
                if (cssProp.isDomCallback()) {
                    const name = cssProp.getDomCallbackName();
                    const callback = objValue;
                    return acm.addDomCallback(new DomCallback(selector, name, callback));
                }
                const macro = new CssMacro(objValue);
                const macroValue = macro.call(selector);
                if (macroValue !== "") {
                    const cssText = new CssText({ prop: cssProp.value, value: macroValue });
                    return acm.setProperty(cssProp.value, cssText.value);
                }
            }
            return acm;
        }, new CssStyleDeclaration());
    }
    static parseDeclaration(cssProp, cssText) {
        switch (cssProp.value) {
            case "border":
                return LogicalBorder.parseShorthand(cssText);
            case "border-before":
                return LogicalBorder.parseShorthandEach(cssText, "before");
            case "border-end":
                return LogicalBorder.parseShorthandEach(cssText, "end");
            case "border-after":
                return LogicalBorder.parseShorthandEach(cssText, "after");
            case "border-start":
                return LogicalBorder.parseShorthandEach(cssText, "start");
            case "border-radius":
                return LogicalBorderRadius.parseShorthand(cssText);
            case "font":
                return Font.parseShorthand(cssText);
            case "list-style":
                return ListStyle.parseShorthand(cssText);
            case "margin":
                return LogicalMargin.parseShorthand(cssText);
            case "padding":
                return LogicalPadding.parseShorthand(cssText);
            case "border-width":
                return LogicalBorderWidth.parseShorthand(cssText);
            case "border-color":
                return LogicalBorderColor.parseShorthand(cssText);
            case "border-style":
                return LogicalBorderStyle.parseShorthand(cssText);
            case "text-emphasis":
                return TextEmphasis.parseShorthand(cssText);
        }
        return [{
                prop: cssProp.value,
                value: cssText.value
            }];
    }
}
//# sourceMappingURL=css-parser.js.map