import { Config, LayoutResult, LogicalNodeGenerator, BlockMargin, InlineMargin, FlowFormatContext, InlineReducer, } from './public-api';
export class InlineNodeGenerator {
    constructor(context, reducer = InlineReducer.instance) {
        this.context = context;
        this.reducer = reducer;
        this.generator = this.createGenerator();
    }
    getNext() {
        const next = this.generator.next();
        return next.done ? undefined : next.value;
    }
    *createGenerator() {
        if (Config.debugLayout) {
            console.group(`inline: ${this.context.name}`);
        }
        let prevChildGen = undefined;
        if (this.context.rootExtent < this.context.contextBoxEdge.borderBoxExtent ||
            this.context.rootMeasure < this.context.contextBoxEdge.borderBoxMeasure) {
            yield LayoutResult.skip(this.context, "Too large edge size: this layout can't be included!");
            return;
        }
        if (this.context.restExtent < this.context.contextBoxEdge.borderBoxExtent) {
            yield LayoutResult.pageBreak(this.context, "inline rest extent not enough for border size");
        }
        this.context.addBorderBoxEdge("before");
        this.context.addBorderBoxEdge("after");
        const startEdgeSize = this.context.contextBoxEdge.getBorderBoxEdgeSize("start");
        if (this.context.restMeasure < startEdgeSize) {
            yield LayoutResult.lineBreak(this.context, "Start edge is not enough for restMeasure");
        }
        this.context.addBorderBoxEdge("start");
        let childElement = this.context.env.element.firstChild;
        while (childElement !== null) {
            const inlineMarginSize = InlineMargin.getMarginFromLastInline(childElement);
            if (inlineMarginSize > 0 && this.context.restMeasure > inlineMarginSize) {
                this.context.addInlineMarginEdge("start", inlineMarginSize);
            }
            const childGen = LogicalNodeGenerator.createChild(childElement, this.context);
            this.context.child = childGen.generator;
            if (this.context.child.context.env.display.isBlockLevel()) {
                if (this.context.inlineNodes.length > 0) {
                    yield this.context.acceptLayoutReducer(this.reducer, true);
                    yield LayoutResult.lineBreak(this.context, "sweep out inlines for next block(inside inline)");
                }
                if (this.context.parentBlock && this.context.parentBlock.localPos.start > 0) {
                    yield LayoutResult.lineBreak(this.context, "line break from inner inline");
                }
            }
            const beforeMargin = BlockMargin.getFlowMarginFromLastElement(this.context.env, this.context.child, prevChildGen);
            const parentBlockCtx = this.context.parentBlock;
            if (parentBlockCtx && beforeMargin > 0) {
                if (parentBlockCtx.restExtent < beforeMargin) {
                    yield LayoutResult.pageBreak(this.context, `block-fmt-context: before-margin(${beforeMargin}) is not enough.`);
                }
                if (parentBlockCtx instanceof FlowFormatContext) {
                    parentBlockCtx.addBlockMarginEdge("before", beforeMargin);
                }
            }
            while (true) {
                const value = this.context.child.getNext();
                if (!value) {
                    break;
                }
                if (value.type === 'skip') {
                    break;
                }
                if (value.type === 'line-break') {
                    yield this.context.acceptLayoutReducer(this.reducer, true);
                    yield value;
                }
                else if (value.type == 'iblock-inline-break') {
                    if (this.context.inlineNodes.length > 0) {
                        yield this.context.acceptLayoutReducer(this.reducer, true);
                        yield LayoutResult.lineBreak(this.context, "inline: iblock-inline-break -> line-break");
                    }
                    else {
                        yield value;
                    }
                }
                else if (value.type === 'page-break') {
                    if (this.context.inlineNodes.length > 0) {
                        yield this.context.acceptLayoutReducer(this.reducer, true);
                        yield value;
                    }
                    else {
                        yield value;
                    }
                }
                else if (value.type === 'inline') {
                    this.context.addInline(value.body);
                }
                else if (value.type === 'text') {
                    const textNode = value.body;
                    if (textNode.skipBr && childGen.nextElement && childGen.nextElement.tagName === "br") {
                        childGen.nextElement = childGen.nextElement.nextSibling;
                    }
                    this.context.addText(textNode);
                }
                else if (value.type === 'ruby') {
                    this.context.addRuby(value.body);
                }
                else if (value.type === 'inline-link') {
                    this.context.addInlineLink(value.body);
                }
                else if (value.type === 're-inline') {
                    this.context.addInlineRe(value.body);
                }
                else if (value.type === 'inline-block') {
                    this.context.addInlineBlock(value.body);
                }
                else if (value.isBlockLevel()) {
                    yield value;
                }
            }
            prevChildGen = childGen.generator;
            childElement = childGen.nextElement;
        }
        const endEdgeSize = this.context.contextBoxEdge.getBorderBoxEdgeSize("end");
        if (endEdgeSize < this.context.restMeasure) {
            this.context.addBorderBoxEdge("end");
        }
        const endMarginSize = this.context.contextBoxEdge.margin.getSize("end");
        if (endMarginSize < this.context.restMeasure) {
            this.context.addInlineMarginEdge("end", endMarginSize);
        }
        yield this.context.acceptLayoutReducer(this.reducer, false);
        if (Config.debugLayout) {
            console.groupEnd();
        }
    }
}
//# sourceMappingURL=inline-node-generator.js.map