import { Config, } from './public-api';
export class LogicalTextNode {
    constructor(env, boxPos, size, text, skipBr, children, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.text = text;
        this.skipBr = skipBr;
        this.children = children;
        this.progress = progress;
        this.dom = dom;
    }
    get measure() {
        return this.size.measure;
    }
    get extent() {
        return this.size.extent;
    }
    acceptEvaluator(visitor) {
        return visitor.visitText(this);
    }
    acceptEffector(visitor) { }
}
export class LogicalLineNode {
    constructor(env, boxPos, linePos, size, autoSize, text, children, baseline, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.linePos = linePos;
        this.size = size;
        this.autoSize = autoSize;
        this.text = text;
        this.children = children;
        this.baseline = baseline;
        this.progress = progress;
        this.dom = dom;
        this.children.forEach(child => child.parent = this);
    }
    get measure() {
        return this.size.measure;
    }
    get extent() {
        return this.size.extent;
    }
    acceptEvaluator(visitor) {
        return visitor.visitLine(this);
    }
    acceptEffector(visitor) {
        visitor.visitLine(this);
    }
}
export class LogicalRubyNode {
    constructor(env, boxPos, size, text, rb, rt, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.text = text;
        this.rb = rb;
        this.rt = rt;
        this.progress = progress;
        this.dom = dom;
    }
    get measure() {
        return this.size.measure;
    }
    get extent() {
        return this.size.extent;
    }
    get lineExtent() {
        const lh = this.env.font.lineHeight;
        if (lh.indexOf("px") < 0) {
            return Math.floor(this.rb.env.font.size * parseFloat(lh));
        }
        return parseInt(lh);
    }
    acceptEvaluator(visitor) {
        return visitor.visitRuby(this);
    }
    acceptEffector(visitor) {
        visitor.visitRuby(this);
    }
}
export class LogicalInlineNode {
    constructor(env, boxPos, size, text, edge, children, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.text = text;
        this.edge = edge;
        this.children = children;
        this.progress = progress;
        this.dom = dom;
        this.children.forEach(child => child.parent = this);
    }
    get measure() {
        return this.size.measure + this.edge.measure;
    }
    get extent() {
        return this.size.extent + this.edge.extent;
    }
    acceptEvaluator(visitor) {
        if (!this.env.textEmphasis.isNone()) {
            return visitor.visitInlineEmpha(this);
        }
        if (this.env.element.tagName === "a") {
            return visitor.visitInlineLink(this);
        }
        return visitor.visitInline(this);
    }
    acceptEffector(visitor) {
        visitor.visitInline(this);
    }
}
export class LogicalBlockNode {
    constructor(env, boxPos, layoutPos, size, autoSize, text, border, children, progress, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.layoutPos = layoutPos;
        this.size = size;
        this.autoSize = autoSize;
        this.text = text;
        this.border = border;
        this.children = children;
        this.progress = progress;
        this.dom = dom;
        this.children.forEach(child => child.parent = this);
    }
    get measure() {
        return this.size.measure + this.env.edge.measure;
    }
    get extent() {
        return this.size.extent + this.border.width.extent;
    }
    acceptEvaluator(visitor) {
        switch (this.env.element.tagName) {
            case "a":
                return visitor.visitBlockLink(this);
            case Config.pageRootTagName:
                const rootBlock = visitor.visitRootBlock(this);
                rootBlock.appendChild(visitor.visitBlock(this));
                return rootBlock;
        }
        return visitor.visitBlock(this);
    }
    acceptEffector(visitor) {
        visitor.visitBlock(this);
    }
}
export class LogicalInlineBlockNode {
    constructor(env, boxPos, layoutPos, size, autoSize, text, edge, children, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.layoutPos = layoutPos;
        this.size = size;
        this.autoSize = autoSize;
        this.text = text;
        this.edge = edge;
        this.children = children;
        this.progress = progress;
        this.dom = dom;
        this.children.forEach(child => child.parent = this);
    }
    get measure() {
        return this.size.measure + this.edge.measure;
    }
    get extent() {
        return this.size.extent + this.edge.extent;
    }
    acceptEvaluator(visitor) {
        return visitor.visitInlineBlock(this);
    }
    acceptEffector(visitor) {
        visitor.visitInlineBlock(this);
    }
}
export class LogicalTableCellsNode {
    constructor(env, boxPos, size, rowPos, text, children, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.rowPos = rowPos;
        this.text = text;
        this.children = children;
        this.progress = progress;
        this.dom = dom;
        this.children.forEach(child => child.parent = this);
    }
    get measure() {
        return this.size.measure;
    }
    get extent() {
        return this.size.extent;
    }
    acceptEvaluator(visitor) {
        return visitor.visitTableCells(this);
    }
    acceptEffector(visitor) {
        visitor.visitTableCells(this);
    }
}
export class LogicalBlockReNode {
    constructor(env, boxPos, size, physicalSize, edge, layoutPos, text, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.physicalSize = physicalSize;
        this.edge = edge;
        this.layoutPos = layoutPos;
        this.text = text;
        this.progress = progress;
        this.dom = dom;
    }
    get measure() {
        return this.size.measure + this.edge.borderBoxMeasure;
    }
    get extent() {
        return this.size.extent + this.env.edge.borderBoxExtent;
    }
    acceptEvaluator(visitor) {
        if (this.env.element.$dom) {
            return visitor.visitBlockReFixed(this, this.env.element.$dom);
        }
        switch (this.env.element.tagName) {
            case "img": return visitor.visitBlockImage(this);
            case "video": return visitor.visitBlockVideo(this);
        }
        console.error("unsupported replaced element:", this);
        throw new Error("unsupported replaced element");
    }
    acceptEffector(visitor) {
        switch (this.env.element.tagName) {
            case "img":
                visitor.visitBlockImage(this);
                break;
            case "video":
                visitor.visitBlockVideo(this);
                break;
        }
    }
}
export class LogicalInlineReNode {
    constructor(env, boxPos, size, physicalSize, edge, text, progress = 1, dom = undefined) {
        this.env = env;
        this.boxPos = boxPos;
        this.size = size;
        this.physicalSize = physicalSize;
        this.edge = edge;
        this.text = text;
        this.progress = progress;
        this.dom = dom;
    }
    get measure() {
        return this.size.measure + this.edge.measure;
    }
    get extent() {
        return this.size.extent + this.edge.extent;
    }
    acceptEvaluator(visitor) {
        if (this.env.element.$dom) {
            return visitor.visitInlineReFixed(this, this.env.element.$dom);
        }
        switch (this.env.element.tagName) {
            case "img": return visitor.visitInlineImage(this);
            case "video": return visitor.visitInlineVideo(this);
        }
        console.error("unsupported replaced element:", this);
        throw new Error("unsupported replaced element");
    }
    acceptEffector(visitor) {
        switch (this.env.element.tagName) {
            case "img":
                visitor.visitInlineImage(this);
                break;
            case "video":
                visitor.visitInlineVideo(this);
                break;
        }
    }
}
//# sourceMappingURL=logical-node.js.map