import { Config, CssLoader, Display, CssLength, CssCascade, CssParser, LogicalEdgeDirections, LogicalBorderRadius, PseudoElement, PseudoElementTagName, ComputedRegion, UsedRegionResolver, LogicalBoxEdge, ListStyle, BorderCollapse, LogicalBorderWidth, } from './public-api';
export class ListItemInitializer {
    constructor() { }
    visit(element) {
        const display = Display.load(element);
        if (!display.isListItem() || display.isNone() || !element.parent) {
            return;
        }
        const listStyle = ListStyle.load(element);
        if (listStyle.isNone()) {
            return;
        }
        let markerText = listStyle.getMarkerText(element.indexOfType);
        const markerElement = element.firstChild;
        if (!markerElement || markerElement.tagName !== PseudoElementTagName.MARKER) {
            throw new Error("marker element is not created yet!");
        }
        const listTextNode = element.ownerDocument.createTextNode(markerText);
        markerElement.appendChild(listTextNode);
    }
}
ListItemInitializer.instance = new ListItemInitializer();
export class TableRowGroupInitializer {
    constructor() { }
    visit(element) {
        const parent = element.parent;
        const borderCollapse = BorderCollapse.load(element);
        if (!borderCollapse.isCollapse() || !parent) {
            return;
        }
        const parentMeasure = parseInt(parent.computedStyle.getPropertyValue("measure") || "0");
        const parentBorderSize = LogicalBorderWidth.load(parent).measure;
        const borderSize = LogicalBorderWidth.load(element).measure;
        const collapsedMeasure = parentMeasure - Math.max(0, borderSize - parentBorderSize);
        element.computedStyle.setProperty("measure", collapsedMeasure + "px");
    }
}
TableRowGroupInitializer.instance = new TableRowGroupInitializer();
export class TableRowInitializer {
    constructor() { }
    visit(element) {
        const parent = element.parent;
        const borderCollapse = BorderCollapse.load(element);
        if (!borderCollapse.isCollapse() || !parent) {
            return;
        }
        const parentMeasure = parseInt(parent.computedStyle.getPropertyValue("measure") || "0");
        const parentBorderSize = LogicalBorderWidth.load(parent).measure;
        const borderSize = LogicalBorderWidth.load(element).measure;
        const collapsedMeasure = parentMeasure - Math.max(0, borderSize - parentBorderSize);
        element.computedStyle.setProperty("measure", collapsedMeasure + "px");
    }
}
TableRowInitializer.instance = new TableRowInitializer();
export class TableCellInitializer {
    constructor() { }
    getCollapsedInternalEdgeSize(cellEdges, parentEdge) {
        let internalEdgeSize = cellEdges.reduce((sum, cellEdge, index) => {
            sum += cellEdge.padding.measure;
            return (index < cellEdges.length - 1) ? sum + Math.max(cellEdge.border.width.end, cellEdges[index + 1].border.width.start) : sum;
        }, 0);
        internalEdgeSize += Math.max(0, cellEdges[0].border.width.start - parentEdge.border.width.start);
        internalEdgeSize += Math.max(0, cellEdges[cellEdges.length - 1].border.width.end - parentEdge.border.width.end);
        return internalEdgeSize;
    }
    getTableCells(element) {
        return element.children.filter(child => {
            return child.computedStyle.getPropertyValue("display") === "table-cell";
        });
    }
    findPrevCells(parent, cellDims) {
        let prevParent = parent.previousElementSibling;
        while (prevParent) {
            const prevCells = this.getTableCells(prevParent);
            if (prevCells.length === cellDims) {
                return prevCells;
            }
            prevParent = prevParent.previousElementSibling;
        }
        return undefined;
    }
    visit(element) {
        const parent = element.parent;
        if (!parent || element.computedStyle.getPropertyValue("display") !== "table-cell") {
            return;
        }
        const cells = this.getTableCells(parent);
        if (cells.every(cell => cell.computedStyle.getPropertyValue("measure") !== "auto")) {
            return;
        }
        if (Display.load(parent).isTableRow()) {
            const prevCells = this.findPrevCells(parent, cells.length);
            if (prevCells) {
                cells.forEach((cell, index) => {
                    const prevMeasure = prevCells[index].computedStyle.getPropertyValue("measure");
                    if (!prevMeasure) {
                        throw new Error("cell partition is not defined properly.");
                    }
                    cell.computedStyle.setProperty("measure", prevMeasure);
                });
                return;
            }
        }
        cells.forEach(cell => {
            LogicalEdgeDirections.forEach(dir => {
                cell.computedStyle.setProperty(`margin-${dir}`, "0");
            });
        });
        const parentEdge = LogicalBoxEdge.load(parent);
        const cellEdges = cells.map(cell => LogicalBoxEdge.load(cell));
        const borderCollapse = BorderCollapse.load(element);
        const internalEdgeSize = borderCollapse.isCollapse() ?
            this.getCollapsedInternalEdgeSize(cellEdges, parentEdge) :
            cellEdges.reduce((sum, cellEdge) => sum + cellEdge.measure, 0);
        const parentMeasure = parseInt(parent.computedStyle.getPropertyValue("measure") || "0", 10);
        const cellMeasures = cells.map(cell => {
            const measure = cell.computedStyle.getPropertyValue("measure") || "0";
            return (measure === "auto") ? 0 : parseInt(measure, 10);
        });
        const autoCells = cells.filter(cell => cell.computedStyle.getPropertyValue("measure") === "auto");
        const fixedSize = cellMeasures.reduce((sum, size) => sum + size, 0);
        const autoMeasure = parentMeasure - fixedSize - internalEdgeSize;
        const autoCellSize = Math.max(Math.floor(autoMeasure / autoCells.length), 0);
        const autoFraction = autoMeasure % autoCells.length;
        if (Config.debugLayout) {
            console.log("cell size:(parent:%d, fixedSize:%d, iedge:%d, auto:%d(fraction:%d)), cell:", parentMeasure, fixedSize, internalEdgeSize, autoCellSize, autoFraction, element);
        }
        autoCells.forEach((cell, index) => {
            const usedAutoSize = (index === 0) ? autoCellSize + autoFraction : autoCellSize;
            cell.computedStyle.setProperty("measure", usedAutoSize + "px");
        });
    }
}
TableCellInitializer.instance = new TableCellInitializer();
export class RubyNormalizer {
    constructor() { }
    visit(element) {
        if (element.tagName !== "ruby") {
            return;
        }
        const doc = element.root;
        element.childNodes = element.childNodes.filter(node => {
            if (node.tagName === "rp") {
                return false;
            }
            if (node.isTextElement() && node.textContent.trim() === "") {
                return false;
            }
            return true;
        });
        element.childNodes = element.childNodes.map(node => {
            if (!node.isTextElement()) {
                return node;
            }
            const rb = doc.createElement("rb");
            rb.parent = element;
            rb.appendChild(node);
            CssLoader.load(rb);
            return rb;
        });
    }
}
RubyNormalizer.instance = new RubyNormalizer();
export class PseudoElementInitializer {
    constructor(pseudoRules) {
        this.pseudoRules = pseudoRules;
    }
    addMarker(element) {
        const markerElement = element.root.createElement(PseudoElementTagName.MARKER);
        markerElement.parent = element;
        element.insertBefore(markerElement, element.firstChild);
        return markerElement;
    }
    addBefore(element) {
        const before = element.root.createElement(PseudoElementTagName.BEFORE);
        element.insertBefore(before, element.firstChild);
        return before;
    }
    addAfter(element) {
        const after = element.root.createElement(PseudoElementTagName.AFTER);
        element.appendChild(after);
        return after;
    }
    addFirstLine(element) {
        const firstLine = element.root.createElement(PseudoElementTagName.FIRST_LINE);
        const firstTextNode = element.firstTextElement;
        if (!firstTextNode) {
            return null;
        }
        const targetParent = firstTextNode.parent;
        if (!targetParent) {
            return null;
        }
        firstLine.appendChild(firstTextNode);
        targetParent.replaceChild(firstLine, firstTextNode);
        return firstLine;
    }
    addFirstLetter(element) {
        const firstTextNode = element.firstTextElement;
        if (!firstTextNode) {
            return null;
        }
        const targetParent = firstTextNode.parent;
        if (!targetParent) {
            return null;
        }
        const text = firstTextNode.textContent;
        const trimText = text.trim();
        const target_text = (trimText.length > 1) ? trimText : text;
        const firstText = target_text.substring(0, 1);
        const nextText = text.substring(1);
        const firstLetter = element.root.createElement(PseudoElementTagName.FIRST_LETTER);
        firstLetter.appendChild(element.root.createTextNode(firstText));
        const nextNode = element.root.createTextNode(nextText);
        nextNode.appendChild(element.root.createTextNode(nextText));
        const baseNode = firstTextNode.nextSibling;
        targetParent.removeChild(firstTextNode);
        targetParent.insertBefore(nextNode, baseNode);
        targetParent.insertBefore(firstLetter, nextNode);
        return firstLetter;
    }
    addPseudoElement(element, peTagName) {
        switch (peTagName) {
            case PseudoElementTagName.MARKER:
                return this.addMarker(element);
            case PseudoElementTagName.BEFORE:
                return this.addBefore(element);
            case PseudoElementTagName.AFTER:
                return this.addAfter(element);
            case PseudoElementTagName.FIRST_LETTER:
                return this.addFirstLetter(element);
            case PseudoElementTagName.FIRST_LINE:
                return this.addFirstLine(element);
        }
        throw new Error("undefined pseudo element:" + peTagName);
    }
    visit(element) {
        this.pseudoRules.forEach(rule => {
            if (rule.test(element, true) && rule.peSelector) {
                const peName = rule.peSelector.tagName;
                const pe = element.querySelector(peName) || this.addPseudoElement(element, peName);
                if (pe) {
                    pe.style.mergeFrom(rule.style);
                }
            }
        });
    }
}
export class CssSpecifiedValueLoader {
    constructor() { }
    visit(element) {
        if (element.isTextElement() || PseudoElement.isPseudoElement(element)) {
            return;
        }
        const specStyle = element.ownerDocument.specStyleSheet.getStyleOfElement(element);
        element.style = specStyle;
    }
}
CssSpecifiedValueLoader.instance = new CssSpecifiedValueLoader();
export class CssSpecifiedDynamicValueLoader {
    constructor(parentCxt) {
        this.parentCxt = parentCxt;
    }
    visit(element) {
        const dynamicStyle = element.style.getDynamicStyle(element, this.parentCxt);
        element.style.mergeFrom(dynamicStyle);
        dynamicStyle.forEach((key, value) => {
            element.computedStyle.removeProperty(key);
        });
    }
}
export class CssSpecifiedInlineValueLoader {
    constructor() { }
    visit(element) {
        const inlineStyleSrc = element.getAttribute("style") || "";
        const inlineStyle = CssParser.parseInlineStyle(inlineStyleSrc);
        element.style.mergeFrom(inlineStyle);
    }
}
CssSpecifiedInlineValueLoader.instance = new CssSpecifiedInlineValueLoader();
export class CssComputedValueLoader {
    getLineHeightString(element) {
        return CssLength.computeLineHeight(element);
    }
    setCascadedValue(element, prop) {
        const value = CssCascade.getValue(element, prop);
        element.computedStyle.setProperty(prop, value);
        return value;
    }
    setFontSize(element) {
        const fontSize = CssLength.computeFontSize(element);
        element.computedStyle.setProperty("font-size", fontSize + "px");
    }
    setLineHeight(element) {
        const lineHeightStr = this.getLineHeightString(element);
        element.computedStyle.setProperty("line-height", lineHeightStr);
    }
    setBoxLength(element, prop) {
        const size = CssLength.computeBoxLength(element, prop);
        element.computedStyle.setProperty(prop, size + "px");
    }
    setAutableBoxLength(element, prop) {
        const size = CssLength.computeAutableBoxLength(element, prop);
        const value = (size === "auto") ? size : size + "px";
        element.computedStyle.setProperty(prop, value);
    }
    setOptionalBoxLength(element, prop) {
        const size = CssLength.computeOptionalBoxLength(element, prop);
        const value = (size === "none") ? size : size + "px";
        element.computedStyle.setProperty(prop, value);
    }
    setMargin(element) {
        LogicalEdgeDirections.forEach(direction => {
            this.setAutableBoxLength(element, `margin-${direction}`);
        });
    }
    setPadding(element) {
        LogicalEdgeDirections.forEach(direction => {
            this.setBoxLength(element, `padding-${direction}`);
        });
    }
    setBorderWidth(element) {
        LogicalEdgeDirections.forEach(direction => {
            const prop = `border-${direction}-width`;
            const size = CssLength.computeBorderWidth(element, prop);
            element.computedStyle.setProperty(prop, size + "px");
        });
    }
    setBorderStyle(element) {
        LogicalEdgeDirections.forEach(direction => {
            const prop = `border-${direction}-style`;
            const value = CssCascade.getValue(element, prop);
            element.computedStyle.setProperty(prop, value);
        });
    }
    setBorderColor(element) {
        LogicalEdgeDirections.forEach(direction => {
            const prop = `border-${direction}-color`;
            const value = CssCascade.getValue(element, prop);
            element.computedStyle.setProperty(prop, value);
        });
    }
    setBorderRadius(element) {
        LogicalBorderRadius.corners.forEach((corner) => {
            const prop = `border-${corner}-radius`;
            const size = CssLength.computeBoxLength(element, prop);
            element.computedStyle.setProperty(prop, size + "px");
        });
    }
    setPosition(element) {
        LogicalEdgeDirections.forEach(direction => {
            this.setAutableBoxLength(element, direction);
        });
    }
    visit(element) {
        if (element.isTextElement()) {
            return;
        }
        const display = this.setCascadedValue(element, "display");
        if (display === "none") {
            return;
        }
        if (Config.nonLayoutTags.includes(element.tagName)) {
            return;
        }
        this.setCascadedValue(element, "writing-mode");
        this.setCascadedValue(element, "color");
        this.setFontSize(element);
        this.setLineHeight(element);
        if (Config.fontSizeOnlyTags.includes(element.tagName)) {
            return;
        }
        if (!Config.edgeSkipTags.includes(element.tagName)) {
            this.setPadding(element);
            this.setBorderWidth(element);
            this.setBorderStyle(element);
            this.setBorderColor(element);
            this.setBorderRadius(element);
        }
        if (!Config.boxSizeSkipTags.includes(element.tagName)) {
            this.setMargin(element);
            this.setPosition(element);
            this.setAutableBoxLength(element, "measure");
            this.setAutableBoxLength(element, "extent");
            this.setAutableBoxLength(element, "width");
            this.setAutableBoxLength(element, "height");
            this.setOptionalBoxLength(element, "min-measure");
            this.setOptionalBoxLength(element, "max-measure");
            this.setOptionalBoxLength(element, "min-extent");
            this.setOptionalBoxLength(element, "max-extent");
        }
        this.setCascadedValue(element, "box-sizing");
        this.setCascadedValue(element, "float");
        this.setCascadedValue(element, "font-family");
        this.setCascadedValue(element, "font-style");
        this.setCascadedValue(element, "font-weight");
        this.setCascadedValue(element, "font-variant");
        this.setCascadedValue(element, "font-stretch");
        this.setCascadedValue(element, "text-orientation");
        this.setCascadedValue(element, "text-combine-upright");
        this.setCascadedValue(element, "text-emphasis-style");
        this.setCascadedValue(element, "text-emphasis-color");
        this.setCascadedValue(element, "text-align");
        this.setCascadedValue(element, "vertical-align");
        this.setCascadedValue(element, "list-style-type");
        this.setCascadedValue(element, "list-style-position");
        this.setCascadedValue(element, "list-style-image");
        this.setCascadedValue(element, "content");
        this.setCascadedValue(element, "word-break");
        this.setCascadedValue(element, "overflow-wrap");
        this.setCascadedValue(element, "white-space");
        this.setCascadedValue(element, "page-break-before");
        this.setCascadedValue(element, "page-break-after");
        this.setCascadedValue(element, "position");
        this.setCascadedValue(element, "border-collapse");
    }
}
CssComputedValueLoader.instance = new CssComputedValueLoader();
export class CssUsedRegionLoader {
    constructor() { }
    visit(element) {
        const display = Display.load(element);
        if (display.isNone()) {
            return;
        }
        if (Config.boxSizeSkipTags.includes(element.tagName)) {
            return;
        }
        const computedRegion = ComputedRegion.load(element);
        if (!computedRegion) {
            return;
        }
        const resolver = UsedRegionResolver.select(element);
        resolver.resolve(element, computedRegion, display);
        computedRegion.save(element.computedStyle);
    }
}
CssUsedRegionLoader.instance = new CssUsedRegionLoader();
//# sourceMappingURL=node-effector.js.map