import { SelectorTokenType, UniversalSelector, TypeSelector, AttrSelector, IdSelector, ClassSelector, PseudoClassSelector, PseudoElementSelector, CompoundSelector, ComplexSelector, } from "./public-api";
export class SelectorParser {
    constructor(lexer) {
        this.lexer = lexer;
    }
    normalizeAttr(src) {
        let norm = src.trim()
            .replace(/^\[/, "")
            .replace(/\]$/, "")
            .replace(/["']/g, "")
            .replace(/\s/g, "");
        return norm;
    }
    parseAttrSelector(src) {
        let body = this.normalizeAttr(src);
        if (body.indexOf("=") < 0) {
            return new AttrSelector(body);
        }
        let operators = ["*=", "^=", "$=", "~=", "|=", "="];
        for (var i = 0; i < operators.length; i++) {
            let operator = operators[i];
            let parts = body.split(operator);
            if (parts.length >= 2) {
                return new AttrSelector(parts[0], operator, parts[1]);
            }
        }
        throw new Error("syntax error(attribute selector)");
    }
    parseCompoundSelector() {
        let loop = true, prop_count = 0;
        let selectors = {
            classSelectors: [],
            pseudoClasses: []
        };
        while (loop && this.lexer.hasNext()) {
            let token = this.lexer.getNext();
            switch (token.type) {
                case SelectorTokenType.UNIVERSAL_SELECTOR:
                    selectors.univSelector = new UniversalSelector();
                    prop_count++;
                    break;
                case SelectorTokenType.TYPE_SELECTOR:
                    selectors.typeSelector = new TypeSelector(token.value);
                    prop_count++;
                    break;
                case SelectorTokenType.ID_SELECTOR:
                    selectors.idSelector = new IdSelector(token.value);
                    prop_count++;
                    break;
                case SelectorTokenType.CLASS_SELECTOR:
                    selectors.classSelectors.push(new ClassSelector(token.value));
                    prop_count++;
                    break;
                case SelectorTokenType.ATTR_SELECTOR:
                    selectors.attrSelector = this.parseAttrSelector(token.value);
                    prop_count++;
                    break;
                case SelectorTokenType.PSEUDO_CLASS:
                    selectors.pseudoClasses.push(new PseudoClassSelector(token.value));
                    prop_count++;
                    break;
                case SelectorTokenType.PSEUDO_ELEMENT:
                    selectors.pseudoElement = new PseudoElementSelector(token.value);
                    prop_count++;
                    break;
                default:
                    this.lexer.pushBack(1);
                    loop = false;
                    break;
            }
        }
        if (prop_count === 0) {
            throw new Error("syntax error. selector required");
        }
        return new CompoundSelector(selectors);
    }
    parseCombinator() {
        let token = this.lexer.getNext();
        if (token.type !== SelectorTokenType.COMBINATOR) {
            throw new Error("syntax error. combinator required.");
        }
        return token.value;
    }
    parse() {
        let selectors = [];
        let combinators = [];
        let right = null;
        while (this.lexer.hasNext()) {
            if (!right) {
                right = this.parseCompoundSelector();
                selectors.push(right);
            }
            if (!this.lexer.hasNext()) {
                break;
            }
            let combinator = this.parseCombinator();
            combinators.push(combinator);
            if (!this.lexer.hasNext()) {
                throw new Error("missing left value for combinator[" + combinator + "]");
            }
            let left = this.parseCompoundSelector();
            selectors.push(left);
            right = left;
        }
        return new ComplexSelector(selectors, combinators);
    }
}
//# sourceMappingURL=selector-parser.js.map