import { Config, LayoutResult, Word, DualChar, SpaceChar, TextReducer, Hyphenator, Kerning, } from './public-api';
export class TextNodeGenerator {
    constructor(context, reducer = TextReducer.instance, hyphenator = Hyphenator.instance, kerning = Kerning.instance) {
        this.context = context;
        this.reducer = reducer;
        this.hyphenator = hyphenator;
        this.kerning = kerning;
        this.generator = this.createGenerator();
    }
    getNext() {
        const next = this.generator.next();
        return next.done ? undefined : next.value;
    }
    *createGenerator() {
        const lexer = this.context.lexer;
        while (this.context.lexer.hasNext()) {
            const font = this.context.env.font;
            const lineExtent = font.lineExtent;
            const empha = this.context.env.textEmphasis.isNone() ? undefined : this.context.env.textEmphasis.textEmphaData;
            const isPre = this.context.env.whiteSpace.isPre();
            const isVertical = this.context.env.writingMode.isTextVertical();
            const metricsArgs = { font, isVertical, empha };
            const isLineHead = this.context.isLineHead();
            if (this.context.restExtent < lineExtent) {
                yield LayoutResult.pageBreak(this.context, "lineExtent is not enough for text-fmt-context");
            }
            if (isLineHead) {
                if (this.context.restMeasure < font.size) {
                    if (this.context.inlineRoot.env.display.isInlineBlockFlow()) {
                        yield LayoutResult.iblockInlineBreak(this.context, "iblock starts from too narrow space");
                    }
                    else {
                        const msg = `too narrow space: restM:${this.context.restMeasure}, start:${this.context.inlineRoot.cursorPos.start}`;
                        yield LayoutResult.skip(this.context, msg);
                        break;
                    }
                }
            }
            const token = lexer.getNext();
            if (Config.debugCharacter) {
                console.log("restM:%d, token:%o", this.context.restMeasure, token);
            }
            if (isPre && token.text === "\n") {
                yield this.context.acceptLayoutReducer(this.reducer, false);
                yield LayoutResult.lineBreak(this.context, "CRLF in white-space:pre");
                continue;
            }
            if (token.size.measure === 0 || token instanceof Word) {
                token.setMetrics(metricsArgs);
            }
            if (token instanceof DualChar && token.isKernEnable()) {
                const prev = lexer.peek(-2);
                if (prev instanceof DualChar && this.kerning.set(token, prev)) {
                    token.setMetrics(metricsArgs);
                }
            }
            if (this.context.restMeasure < token.size.measure) {
                if (token instanceof Word) {
                    if (isLineHead && this.context.env.overflowWrap.isBreakWord()) {
                        lexer.pushBack(1);
                        this.context.addCharacter(token.breakWord(this.context.restMeasure));
                    }
                    else if (this.context.env.wordBreak.isBreakAll()) {
                        lexer.pushBack(1);
                        this.context.addCharacter(token.breakWord(this.context.restMeasure));
                    }
                    else if (isLineHead) {
                        this.context.addCharacter(token);
                    }
                    else {
                        lexer.pushBack(1);
                        this.hyphenator.hyphenate(this.context);
                    }
                    yield this.context.acceptLayoutReducer(this.reducer, true);
                    yield LayoutResult.lineBreak(this.context, "long word is broken by word-break: break-all");
                }
                else if (token instanceof SpaceChar) {
                    const prevToken = lexer.peek(-2);
                    const nextToken = lexer.peek(0);
                    if (!(prevToken instanceof Word && nextToken instanceof Word)) {
                        lexer.pushBack(1);
                    }
                    this.hyphenator.hyphenate(this.context);
                    yield this.context.acceptLayoutReducer(this.reducer, true);
                    yield LayoutResult.lineBreak(this.context, "char token overflows measure(by spaceChar)");
                }
                else {
                    lexer.pushBack(1);
                    this.hyphenator.hyphenate(this.context);
                    if (this.context.characters.length > 0) {
                        yield this.context.acceptLayoutReducer(this.reducer, true);
                    }
                    yield LayoutResult.lineBreak(this.context, "char token overflows measure");
                }
            }
            else {
                this.context.addCharacter(token);
            }
        }
        if (this.context.characters.length > 0) {
            yield this.context.acceptLayoutReducer(this.reducer, false);
        }
    }
}
//# sourceMappingURL=text-node-generator.js.map